/**
 * @file eink_hal.h
 * @brief                   EINK click
 *
 ******************************************************************************/

#ifndef EINK_HAL_H_
#define EINK_HAL_H_

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

/* 			Functions
 ******************************************************************************/
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief HAL Initialization
 *
 * Initialization of HAL layer.
 * Initializes function pointers and must be called first.
 */
void hal_eink_init( void );

/**
 * @brief HAL Delay
 *
 * @param[in]   ms          Amount of delay in miliseconds
 */
void hal_eink_delay( uint16_t ms );

/**
 * @brief SPI Write
 *
 * @param[in] 	*buffer     Data for writting
 * @param[in] 	count       Number of bytes to write
 *
 * @par
 * Writes data through SPI bus
 *
 * @note Function have no affect to the CS PIN state - chip select is
 * controled directly from HW layer.
 */
void hal_eink_write
(
        uint8_t* buffer,
        uint16_t count
);

/**
 * @brief SPI Read
 *
 * @param[out] 	*buffer     Buffer to store data from device
 * @param[in] 	count       Number of bytes to read
 *
 * @note Function have no affect to the CS PIN state - chip select is
 * controled directly from HW layer
 */
void hal_eink_read
(
        uint8_t* buffer,
        uint16_t count
);

/**
 * @brief SPI Transfer
 *
 * @param[in] 	*input      Data for writting
 * @param[out] 	*output     Buffer to store data from device
 * @param[in] 	count       Number of bytes to transfer
 *
 * @note Function have no affect to the CS PIN state - chip select is
 * controled directly from HW layer
 *
 */
void hal_eink_transfer
(
        uint8_t* input,
        uint8_t* output,
        uint16_t count
);

#ifdef __cplusplus
} // extern "C"
#endif
#endif
///@}
/**
 * @}                                                           End of File
 ******************************************************************************/
